"""Sphinx conf.py template."""
CONF_PY = '''
"""Configure sphinx for package doc publication."""
import os

import jgt_tools.utils

# General information about your project.
project = "{name}"
copyright = "{year}, {authors}"  # noqa
author = "{authors}"
release = version = "{version}"

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    {type_hints}
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.ifconfig",
    "sphinx.ext.viewcode",
    "sphinx.ext.githubpages",
    "sphinx.ext.inheritance_diagram",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

html_theme_options = {{
    "style_external_links": True,
    "titles_only": False,
    "collapse_navigation": False,
}}

{markdown}

# The master toctree document.
master_doc = "index"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    ".tox",
    "*/.tox",
    ".eggs",
    "*/.eggs",
]

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = "sphinx"

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True

# Include both class and init docstrings
autoclass_content = "both"


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#

# If anyone wants to use another theme, they can change that here,
# but we consider that expert Sphinx user territory.
import sphinx_rtd_theme  # noqa

html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]


commit_id = os.environ.get("ghprbPullId") or os.environ.get("GIT_COMMIT_ID")
base_url = os.environ.get("ghprbPullLink") or ""
if not base_url:
    owner_name = jgt_tools.utils.owner_name_from(os.environ.get("GIT_ORIGIN_URL", ""))
    if owner_name:
        base_url = f"https://github.com/{{owner_name}}/tree/{{commit_id}}"
html_context = {{"build_id": commit_id, "build_url": base_url}}
'''

MARKDOWN = """
from recommonmark.parser import CommonMarkParser  # noqa

source_parsers = {".md": CommonMarkParser}
source_suffix = [".rst", ".md"]
"""

TYPE_HINTS = '"sphinx_autodoc_typehints",'



"""Build the documentation for a package."""
import argparse
import email.utils
import itertools
import os
from pathlib import Path
import shutil
import subprocess
import time

from ..utils import CONFIGS, get_pyproject_config
from .sample_conf import CONF_PY, MARKDOWN, TYPE_HINTS


BASE_DIR = CONFIGS["base_dir"]
PACKAGE_NAME = CONFIGS["package_name"].replace("-", "_").replace(".", "_")
DOC_BUILD_TYPES = CONFIGS["doc_build_types"]

DOCS_OUTPUT_DIRECTORY = "docs"
DOCS_WORKING_DIRECTORY = "_docs"


def _api():
    sphinx_apidoc_cmd = [
        "poetry",
        "run",
        "sphinx-apidoc",
        "--output-dir",
        DOCS_WORKING_DIRECTORY,
        "--no-toc",
        "--force",
        "--module-first",
    ]
    print(f"Building {PACKAGE_NAME} API docs")
    subprocess.check_call(sphinx_apidoc_cmd + [PACKAGE_NAME], cwd=BASE_DIR)


def _build_conf():
    docs_dir = BASE_DIR / DOCS_WORKING_DIRECTORY
    docs_dir.mkdir(exist_ok=True)

    pyproject = get_pyproject_config()
    poetry = pyproject["tool"]["poetry"]

    dependencies = list(
        itertools.chain(poetry["dependencies"], poetry["dev-dependencies"])
    )

    conf = CONF_PY.format(
        name=f"{poetry['name']} - {poetry['description']}",
        authors=", ".join(email.utils.parseaddr(x)[0] for x in poetry["authors"]),
        year=time.strftime("%Y"),
        type_hints=TYPE_HINTS if "sphinx-autodoc-typehints" in dependencies else "",
        version=poetry["version"],
        markdown=MARKDOWN if "recommonmark" in dependencies else "",
    )

    (docs_dir / "conf.py").write_text(conf)


BUILDERS = {"api": _api}


def build():
    """Build the docs."""
    # Setup environment variables
    try:
        commit_id = subprocess.check_output(
            ["git", "rev-parse", "HEAD"],
            cwd=BASE_DIR,
            universal_newlines=True,
            stderr=subprocess.DEVNULL,
        )
    except subprocess.CalledProcessError:
        commit_id = ""
    os.environ["GIT_COMMIT_ID"] = commit_id.rstrip("\n")

    try:
        origin_url = subprocess.check_output(
            ["git", "config", "--get", "remote.origin.url"],
            cwd=BASE_DIR,
            universal_newlines=True,
            stderr=subprocess.DEVNULL,
        )
    except subprocess.CalledProcessError:
        origin_url = ""
    os.environ["GIT_ORIGIN_URL"] = origin_url.rstrip("\n")

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--dirty",
        action="store_true",
        help="Leave the output directory untouched before starting to build documents",
    )
    args = parser.parse_args()

    if not args.dirty:
        shutil.rmtree(str(BASE_DIR / DOCS_OUTPUT_DIRECTORY), ignore_errors=True)
        shutil.rmtree(str(BASE_DIR / DOCS_WORKING_DIRECTORY), ignore_errors=True)

    _build_conf()

    for builder in DOC_BUILD_TYPES:
        if builder not in BUILDERS:
            print(f'No builder for value "{builder}" defined!')
            exit(1)
        BUILDERS[builder]()

    # Copy over all the top level rST files so we don't
    # have to keep a duplicate list here.
    for filename in Path().glob("*.rst"):
        shutil.copy(filename, DOCS_WORKING_DIRECTORY)

    shutil.copy(
        BASE_DIR / ".jgt_tools.index", BASE_DIR / DOCS_WORKING_DIRECTORY / "index.rst"
    )

    os.environ["PYTHONPATH"] = str(Path.cwd())
    subprocess.check_call(
        [
            "poetry",
            "run",
            "sphinx-build",
            "-c",
            DOCS_WORKING_DIRECTORY,
            "-aEW",
            DOCS_WORKING_DIRECTORY,
            DOCS_OUTPUT_DIRECTORY,
        ],
        cwd=BASE_DIR,
    )


def push():
    """Push docs to github-pages."""
    subprocess.check_call(["poetry", "run", "ghp-import", "-p", "docs/"])


def build_and_push():
    """Build docs then publish."""
    build()
    push()



"""Shared util functions."""
from collections import defaultdict
import csv
from pathlib import Path
import shlex
import subprocess
import sys

import tomlkit


DEFAULTS_FILE = Path(__file__).parent / "data" / "defaults.csv"


def execute_command_list(commands_to_run, verbose=True):
    """
    Execute each command in the list.

    If any command fails, print a helpful message and exit with that status.
    """
    for command in commands_to_run:
        if verbose:
            print(f"+{command}")
        job = subprocess.run(shlex.split(command))
        if job.returncode:
            sys.exit(job.returncode)


_DEFAULT_CONFIGS: defaultdict = defaultdict(list)


def _load_defaults():
    with DEFAULTS_FILE.open() as f:
        for group, cmd in csv.reader(f):
            _DEFAULT_CONFIGS[group].append(cmd)


_load_defaults()


def _get_package_root():
    try:
        return Path(
            subprocess.check_output(
                ["git", "rev-parse", "--show-toplevel"], universal_newlines=True
            ).split("\n")[0]
        )
    except subprocess.CalledProcessError:
        print("jgt_tools needs to be run from within a git repository")
        sys.exit(1)


PACKAGE_ROOT_PATH = _get_package_root()


def get_pyproject_config():
    """Get the config data from the project's pyproject.toml file."""
    pyproject_path = PACKAGE_ROOT_PATH / "pyproject.toml"
    if not pyproject_path.exists():
        raise FileNotFoundError(
            f"Config file not found at: '{pyproject_path}' "
            "(must be run from project root)"
        )
    with pyproject_path.open() as f:
        pyproject = tomlkit.loads(f.read())
    return pyproject


def load_configs():
    """Build configs from defaults and pyproject.toml."""
    pyproject = get_pyproject_config()

    poetry = pyproject["tool"]["poetry"]
    package_name = poetry["name"]
    package_description = poetry["description"]

    configs = {
        **_DEFAULT_CONFIGS,
        "package_name": package_name,
        "description": package_description,
        "base_dir": PACKAGE_ROOT_PATH,
    }
    if "jgt_tools" in pyproject["tool"]:
        configs = {**configs, **pyproject["tool"]["jgt_tools"]}
    return configs


CONFIGS = load_configs()


def owner_name_from(origin_url):
    """
    Extract the owner name from a git origin URL.

    The git origin URL might be in ``git+ssh`` form, or ``https`` form.

    Args:
        origin_url (str): Origin URL from `git`

    Returns:
        str: A slash-separated string containing the organization / owner and repository

    Examples:
        >>> owner_name_from("git@github.com:jolly-good-toolbelt/jgt_tools.git")
        "jolly-good-toolbelt/jgt_tools"
        >>> owner_name_from("https://github.com/jolly-good-toolbelt/jgt_tools.git")
        "jolly-good-toolbelt/jgt_tools"

    """
    if not origin_url:
        return ""
    owner_name = origin_url.split(":")[1]  # Remove method portion
    owner_name = owner_name.rsplit(".", 1)[0]  # Remove `.git`
    # Keep only the last two parts that remain, which are the org/owner and repo name
    return "/".join(owner_name.split("/")[-2:])
